@@ -704,7 +704,6 @@ anogva <- function(graphs, labels, numBoot=1000, bandwidth="Silverman") {
   }
   distOrig <- distOrig / max(labels)
 
-
   ## Permutation test
   distBoot <- array(0, numBoot)
 
@@ -743,34 +742,45 @@ anogva <- function(graphs, labels, numBoot=1000, bandwidth="Silverman") {
 #' and 1s. For weighted graphs, each matrix contains real values that correspond
 #' to the weights of the edges.
 #'
-#' @param model A string that indicates one of the following models: "ER" (Erdös-Rényi random graph model),
-#' "GRG" (geometric random graph model), "WS" (Watts-Strogatz random graph model),
-#' and "BA" (Barabási-Albert random graph model).
+#' @param model A string that indicates one of the following models: "ER" 
+#' (Erdös-Rényi random graph model), "GRG" (geometric random graph model), "WS" 
+#' (Watts-Strogatz random graph model), and "BA" (Barabási-Albert random graph 
+#' model).
 #'
-#' @param maxBoot integer indicating the number of bootstrap resamples (default is 500).
+#' @param maxBoot integer indicating the number of bootstrap resamples (default 
+#' is 500).
 #'
 #' @param spectra optional parameter containing the precomputed spectrum of the
 #' model. It is a three-dimensional array in which the first dimension
-#' corresponds to all parameters that will be explored in the parameter estimation,
-#' the second dimension has the same size of the given graph, and the third one
-#' corresponds to graphs randomly generated by the model. Thus, the position (i,j,k)
-#' contains the j-th eigenvalue of the k-th graph generated with the i-th
-#' parameter. The attribute 'rownames' of the array corresponds to the
-#' parameters converted to string. If spectra is NULL (default), then 'model' is
-#' used to generate random graphs and their spectra are computed automatically.
+#' corresponds to all parameters that will be explored in the parameter 
+#' estimation, the second dimension has the same size of the given graph, and 
+#' the third one corresponds to graphs randomly generated by the model. Thus, 
+#' the position (i,j,k) contains the j-th eigenvalue of the k-th graph generated 
+#' with the i-th parameter. The attribute 'rownames' of the array corresponds to 
+#' the parameters converted to string. If spectra is NULL (default), then 
+#' model' is used to generate random graphs and their spectra are computed 
+#' automatically.
 #'
 #' @param eps (default is 0.01) precision of the grid when 'classic' = TRUE.
 #'
-#' @param classic logical. If FALSE (default) parameter is estimated using ternary search,
-#' if TRUE parameter is estimated using grid search.
+#' @param classic logical. If FALSE (default) parameter is estimated using 
+#' ternary search, if TRUE parameter is estimated using grid search.
 #'
 #' @return A list containing:
-#' \item{parameter}{an array containing the estimated parameters for each graph.}
+#' \item{parameter}{an array containing the estimated parameters for each 
+#' graph.}
 #' \item{F.value}{the F statistic of the test.}
 #' \item{p.value}{the p-value of the test.}
 #'
 #' @keywords semi_parametric_analysis_of_graph_variability
 #'
+#' @references
+#'
+#' Andre Fujita, Eduardo Silva Lira, Suzana de Siqueira Santos, Silvia Yumi 
+#' Bando, Gabriela Eleuterio Soares, Daniel Yasumasa Takahashi. A 
+#' semi-parametric statistical test to compare complex networks, Journal of 
+#' Complex Networks, cnz028, https://doi.org/10.1093/comnet/cnz028
+#'
 #' @examples
 #'
 #' ## Please uncomment the following lines to run an example
@@ -799,10 +809,14 @@ sp.anogva <- function(graph, model, maxBoot=500, spectra = NULL, eps = 0.01, cla
   p.hat <- list()
   for(l in 1:g) {
     n <- as.character(ncol(graph[[l]]))
-    if(class(spectra) == "list") p.hat[[l]] <- graph.param.estimator(graph[[l]], model, eps=eps, spectra = spectra[[n]], classic = classic)$p
-    else p.hat[[l]] <- graph.param.estimator(graph[[l]], model, eps=eps, spectra = spectra, classic = classic)$p
+    if (class(spectra) == "list") 
+      p.hat[[l]] <- graph.param.estimator(graph[[l]], model, eps=eps, 
+                                          spectra = spectra[[n]], 
+                                          classic = classic)$p
+    else p.hat[[l]] <- graph.param.estimator(graph[[l]], model, eps=eps, 
+                                             spectra = spectra, 
+                                             classic = classic)$p
   }
-
   p.boot <- matrix(0, length(graph), maxBoot)
   for (boot in 1:maxBoot) {
     g.boot <- list()
@@ -820,18 +834,19 @@ sp.anogva <- function(graph, model, maxBoot=500, spectra = NULL, eps = 0.01, cla
       if (model == "BA") {
         g.boot[[l]] <- BA(ncol(graph[[l]]), p.hat[[l]])
       }
-
-      if(class(spectra) == "list") p.boot[l,boot] <- graph.param.estimator(g.boot[[l]], model, eps=eps, spectra = spectra[[n]], classic = classic)$p
-      else p.boot[l,boot] <- graph.param.estimator(g.boot[[l]], model, eps=eps, spectra = spectra, classic = classic)$p
-
+      if(class(spectra) == "list")
+        p.boot[l,boot] <- graph.param.estimator(g.boot[[l]], model, eps=eps, 
+                                                spectra = spectra[[n]], 
+                                                classic = classic)$p
+      else p.boot[l,boot] <- graph.param.estimator(g.boot[[l]], model, eps=eps, 
+                                                   spectra = spectra, 
+                                                   classic = classic)$p
     }
   }
-
   var.boot <- array(0, g)
   for(l in 1:g) {
     var.boot[l] <- var(p.boot[l,])
   }
-
   SSres <- 0
   SStr <- 0
   m <- mean(as.numeric(as.array(p.hat)))
@@ -839,11 +854,8 @@ sp.anogva <- function(graph, model, maxBoot=500, spectra = NULL, eps = 0.01, cla
     SSres <- SSres + (maxBoot-1) * var.boot[l]
     SStr <- SStr + (p.hat[[l]] - m)^2
   }
-
   F_ <- (SStr / (g-1)) / ( SSres / (g*maxBoot - g))
-
   p <- pf(F_, df1=(g-1), df2=(g*maxBoot - g), lower.tail=FALSE)
-
   res <- list()
   res$parameter <- as.array(p.hat)
   res$F.value <- F_
@@ -1144,22 +1156,28 @@ graph.mult.scaling <- function(x, plot=TRUE, bandwidth="Silverman", type="n",
 
 #' Tang hypothesis testing for random graphs.
 #'
-#' Given two independent finite-dimensional random dot product graphs, 'tang' tests if they have generating latent positions that are
-#' drawn from the same distribution.
+#' Given two independent finite-dimensional random dot product graphs, 'tang' 
+#' tests if they have generating latent positions that are drawn from the same 
+#' distribution.
 #'
-#' @param G1 the first undirected graph to be compared. Must be an igraph object.
+#' @param G1 the first undirected graph to be compared. Must be an igraph 
+#' object.
 #'
-#' @param G2 the second undirected graph to be compared. Must be an igraph object.
+#' @param G2 the second undirected graph to be compared. Must be an igraph 
+#' object.
 #'
 #' @param dim dimension of the adjacency spectral embedding.
 #'
-#' @param sigma a real value indicating the kernel bandwidth. If NULL (default) the bandwidth is calculated by the method.
+#' @param sigma a real value indicating the kernel bandwidth. If NULL (default) 
+#' the bandwidth is calculated by the method.
 #'
 #' @param alpha the significance level for the test (default is 0.05).
 #'
-#' @param bootstrap_sample integer indicating the number of bootstrap resamples (default is 200).
+#' @param bootstrap_sample integer indicating the number of bootstrap resamples 
+#' (default is 200).
 #'
-#' @param printResult logical indicating if the test must print the result (default is FALSE).
+#' @param printResult logical indicating if the test must print the result 
+#' (default is FALSE).
 #'
 #' @return A list containing:
 #' \item{X1}{the embedding of G1.}
@@ -1169,9 +1187,12 @@ graph.mult.scaling <- function(x, plot=TRUE, bandwidth="Silverman", type="n",
 #' \item{bootstrap_samples}{The test distrition on the bootstrap resamples.}
 #'
 #' @references
-#' Tang, Minh, et al. "A nonparametric two-sample hypothesis testing problem for random graphs." Bernoulli 23.3 (2017): 1599-1630.
+#' Tang, Minh, et al. "A nonparametric two-sample hypothesis testing problem for 
+#' random graphs." Bernoulli 23.3 (2017): 1599-1630.
 #'
-#' Tang, Minh, et al. "A semiparametric two-sample hypothesis testing problem for random graphs." Journal of Computational and Graphical Statistics 26.2 (2017): 344-354.
+#' Tang, Minh, et al. "A semiparametric two-sample hypothesis testing problem 
+#' for random graphs." Journal of Computational and Graphical Statistics 26.2 
+#' (2017): 344-354.
 #'
 #' @examples
 #' require(igraph)
@@ -1192,7 +1213,8 @@ graph.mult.scaling <- function(x, plot=TRUE, bandwidth="Silverman", type="n",
 #'
 #'
 #' @export
-tang <- function(G1, G2, dim, sigma = NULL, alpha = 0.05, bootstrap_sample = 200, printResult = FALSE){
+tang <- function(G1, G2, dim, sigma = NULL, alpha = 0.05, bootstrap_sample=200, 
+                 printResult = FALSE){
   t.validateInput(G1, G2, dim, alpha, bootstrap_sample, printResult)
   Xhat1 = t.embed.graph(G1, dim)
   Xhat2 = t.embed.graph(G2, dim)
@@ -1211,9 +1233,13 @@ tang <- function(G1, G2, dim, sigma = NULL, alpha = 0.05, bootstrap_sample = 200
   }
   if (printResult) {
     if (reject) {
-      print("Reject the nullhypothesis that two graphs are identically distributed.")
+      msg <- paste("Reject the nullhypothesis that two graphs are identically", 
+                   "distributed.")
+      print(msg)
     } else {
-      print("Fail to reject the null hypothesis that two graphs are identically distributed.")
+      msg <- paste("Fail to reject the null hypothesis that two graphs are", 
+                   "identically distributed.")
+      print(msg)
     }
   }
   out = list(X1 = Xhat1, X2 = Xhat2, test_stats = test_stat,
@@ -1321,6 +1347,196 @@ ghoshdastidar <- function(x, y, maxPer = 300, alpha = 0.05, two.sample = FALSE,
   return(out)
 }
 
+#' Andressa Cerqueira, Daniel Fraiman, Claudia D. Vargas and Florencia Leonardi 
+#' non-parametric test of hypotheses to verify if two samples of random graphs 
+#' were originated from the same probability distribution.
+#'
+#' Given two identically independently distributed (idd) samples of graphs g and 
+#' gp, the test verifies if they have the same distribution by calculating the 
+#' mean distance D from g to gp. The test rejects the null hypothesis if D is
+#' greater than the (1-alpha)-quantile of the distribution of the test under the 
+#' null hypothesis.
+#'
+#' @param g the first iid sample of graphs to be compared. Must be a list of 
+#' igraph objects.
+#'
+#' @param gp the second iid sample of graphs to be compared. Must be a list of 
+#' igraph objects.
+#'
+#' @param maxPer integer indicating the number of bootstrap resamples (default 
+#' is 300).
+#'
+#' @param alpha the significance level for the test (default is 0.05).
+#'
+#' @param printResult logical indicating if the test must print the result 
+#' (default is FALSE).
+#'
+#' @return A list containing:
+#' \item{test_stats}{the value of the test.}
+#' \item{p_value}{the p-value of the test.}
+#' \item{reject_threshold}{The 1-alpha quantile of the test distribution under 
+#' the null hypothesys.}
+#' \item{bootstrap_samples}{The test distrition on the bootstrap resamples.}
+#'
+#' @references
+#' Andressa Cerqueira, Daniel Fraiman, Claudia D. Vargas and Florencia Leonardi. 
+#' "A test of hypotheses for random graph distributions built from EEG data",
+#' https://ieeexplore.ieee.org/document/7862892
+#'
+#' @examples
+#' require(igraph)
+#' set.seed(42)
+#'
+#' ## test under H0
+#' a <- b <- list()
+#' for(i in 1:10){
+#'   a[[i]] <- erdos.renyi.game(50,0.5)
+#'   b[[i]] <- erdos.renyi.game(50,0.5)
+#' }
+#' k <- cerqueira(a, b, printResult = TRUE)
+#'
+#' ## test under H1
+#' a <- b <- list()
+#' for(i in 1:10){
+#'   a[[i]] <- erdos.renyi.game(50,0.5)
+#'   b[[i]] <- erdos.renyi.game(50,0.6)
+#' }
+#' k <- cerqueira(a, b, printResult = TRUE)
+#'
+#' @export
+cerqueira <- function(g, gp, maxPer = 300, alpha = 0.05, printResult = FALSE)
+{
+  
+  if(class(g)=="list" && class(g[[1]])=="igraph"){
+    g <- c.transform(g)
+  }
+  else{
+    stop("Parameter g must be an a list of igraph objects.")
+  }
+  
+  if(class(gp)=="list" && class(gp[[1]])=="igraph"){
+    gp <- c.transform(gp)
+  }
+  else{
+    stop("Parameter gp must be an a list of igraph objects.")
+  }
+  
+  D <- c.test(g,gp)
+  
+  test_distribution <- c.sampling.distribution(g,gp,maxPer)
+  
+  p_val <- mean(test_distribution >= D)
+  
+  reject_threshold = quantile(test_distribution,1-alpha)
+  
+  if (printResult) {
+    if( D > reject_threshold){
+      msg <- paste("Reject the nullhypothesis that two graphs are identically",
+                   "distributed.")
+      print(msg)
+    } else {
+      msg <- paste("Fail to reject the null hypothesis that two graphs are", 
+                   "identically distributed.")
+      print(msg)
+    }
+  }
+  
+  out = list(test_stats=D, p_value=p_val, reject_threshold=reject_threshold, 
+             bootstrap_samples=test_distribution)
+  
+  return(out)
+}
+
+
+#' Daniel Fraiman and Ricardo Fraiman test for network differences between 
+#' groups with an analysis of variance test (ANOVA).
+#'
+#' Given a list of graphs, the test verifies if all the subpopulations have the 
+#' same mean network,
+#' under the alternative that at least one subpopulation has a different mean 
+#' network.
+#'
+#' @param g the undirected graphs to be compared. Must be a list of lists of 
+#' igraph objects or a list of lists of adjacency matrices.
+#'
+#' @param maxPer integer indicating the number of bootstrap resamples 
+#' (default is 300).
+#'
+#' @param alpha the significance level for the test (default is 0.05).
+#'
+#' @param printResult logical indicating if the test must print the result 
+#' (default is FALSE).
+#'
+#' @return A list containing:
+#' \item{test_stats}{the value of the test.}
+#' \item{p_value}{the p-value of the test.}
+#' \item{bootstrap_samples}{The test distrition on the bootstrap resamples.}
+#'
+#' @references
+#' Fraiman, Daniel, and Ricardo Fraiman. "An ANOVA approach for statistical 
+#' comparisons of brain networks", 
+#' https://www.nature.com/articles/s41598-018-23152-5
+#'
+#' @examples
+#' require(igraph)
+#' set.seed(42)
+#'
+#' ## test under H0
+#' a <- b <- d <- list()
+#' for(i in 1:10){
+#'   a[[i]] <- erdos.renyi.game(50,0.5)
+#'   b[[i]] <- erdos.renyi.game(50,0.5)
+#' }
+#' d <- list(a,b)
+#' k <- fraiman(d, printResult = TRUE)
+#'
+#' ## test under H1
+#' a <- b <- d <- list()
+#' for(i in 1:10){
+#'   a[[i]] <- erdos.renyi.game(50,0.5)
+#'   b[[i]] <- erdos.renyi.game(50,0.6)
+#' }
+#' d <- list(a,b)
+#' k <- fraiman(d, printResult = TRUE)
+#'
+#' @export
+fraiman <- function(g, maxPer = 300, alpha = 0.05, printResult = FALSE){
+  # transform and verify input
+  if(class(g)=='list' && class(g[[1]])=='list' && class(g[[1]][[1]])=='igraph'){
+    g <- f.transform(g)
+  }
+  if(class(g)!='list'|| class(g[[1]])!='list' || class(g[[1]][[1]])!='matrix') 
+    stop(paste("You must pass a list of lists of igraphs or a list of lists of", 
+               "matrices."))
+  
+  D <- f.test(g)
+  
+  test_distribution <- f.sampling.distribution(g, maxPer)
+  
+  ## modification made in april 1st, 2019:
+  p_val <- mean(test_distribution <= D)
+  
+  ## original was:
+  # p_val <- mean(abs(test_distribution) >= abs(D))
+  
+  if (printResult) {
+    if (p_val <= alpha) {
+      print(paste("Reject the nullhypothesis that two graphs are identically", 
+                  "distributed."))
+    }
+    else {
+      print(paste("Fail to reject the null hypothesis that two graphs are", 
+                  "identically distributed."))
+    }
+  }
+  
+  out = list(test_stats=D, p_value=p_val, bootstrap_samples=test_distribution)
+  
+  return(out)
+}
+
+
+
 ################################################################################
 ## Auxiliar functions
 ################################################################################
@@ -1537,6 +1753,152 @@ WSfun <- function(K){
   return(f)
 }
 
+## Auxiliary for Cerqueira method. Test distribution under the null hypothesis
+c.sampling.distribution <- function(g, gp, maxPer = 300)
+{
+  
+  m <- nrow(g)+nrow(gp)
+  test_distribution = c()
+  for (i_per in 1:maxPer){
+    total <- rbind(g, gp)
+    ind <- sample(1:m, floor(m/2), replace=F)
+    xa <- total[ind,]
+    ya <- total[-ind,]
+    test_distribution[i_per] <- c.test(xa,ya)
+  }
+  return(sort(test_distribution))
+}
+
+## Auxiliary for Cerqueira method. Fix input format.
+c.transform <- function(g, n = igraph::gorder(g[[1]]))
+{
+  x <- matrix(0, length(g), n*(n-1)/2)
+  i <- 1
+  for(gr in g){
+    aux <- as.matrix(igraph::get.adjacency(gr))
+    x[i,] <- aux[upper.tri(aux)]
+    i <- i+1
+  }
+  return(x)
+}
+
+## Auxiliary for Cerqueira method. The test itself.
+c.test <- function(g, gp)
+{
+  wstat <- sum(abs(colMeans(g)-colMeans(gp)))
+  return(wstat)
+}
+
+# Auxiliary for Fraiman method. Fraiman test itself according to article.
+f.test <- function(g){
+  # we need to create a function to calculate a [it's complicated - Appendix 1.3], so far we are using this
+  a <- 1
+  
+  #how many sets we have
+  m <- length(g)
+  
+  #size of each set
+  l <- unlist(lapply(g, length))
+  
+  # make g upper triangular
+  g <- lapply(g, f.upper)
+  
+  # matrix of mean matrices Mi's
+  M <- f.calcM(g)
+  
+  # create a list G with all the graphs
+  G <- list()
+  for(i in 1:length(g)) G <- append(G, g[[i]])
+  
+  # calculates $\bar{d}_G(\mathcal{M}_i)$
+  # the distance from each $\mathcal{M}_i$ to the entire set of graphs
+  sumDG <- rep(0, length(M))
+  for(i in 1:length(M)){
+    for(j in 1:length(G)){
+      sumDG[i] <- sumDG[i] + sum(abs(G[[j]]-M[[i]]))
+    }
+    sumDG[i] <- sumDG[i]/length(G)
+  }
+  
+  # calculates $\bar{d}_{G^i}(\mathcal{M}_i)$
+  # the distance from each $\mathcal{M}_i$ to the set i of graphs
+  sumDGi <- rep(0, length(M))
+  
+  for(i in 1:length(M)){
+    for(j in 1:length(g[[i]])){
+      sumDGi[i] <- sumDGi[i] + sum(abs(g[[i]][[j]] - M[[i]]))
+    }
+    sumDGi[i] <- sumDGi[i]/length(g[[i]])
+  }
+  
+  # calculates the final value of the test (equation 2.3)
+  # T := \frac{\sqrt(m)}{a} \sum\limits_{i=1}^m \sqrt(n_i) \left( \frac{n_i}{n_i-1} \bar{d}_{G^i}(\mathcal{M_i}) - \frac{n}{n-1}\bar{d}_G(\mathcal{M}_i) \right)
+  t1 <- (l/(l-1))*sumDGi
+  t2 <- (sum(l)/(sum(l)-1))*sumDG
+  t <- (sqrt(m)/a)*sum(sqrt(l)*(t1-t2))
+  
+  return(t)
+}
+
+# Auxiliary for Fraiman method. Intends to speed calculations using R builtins.
+f.upper <- function(x) lapply(x, function(s){
+  s2 <- s
+  s2[lower.tri(s2)]<-0
+  eval.parent(substitute(s<-s2))
+})
+
+# Auxiliary for Fraiman method. Intends to speed calculations using R builtins.
+f.add <- function(x){ list(Reduce("+", x), length(x)) }
+
+# Auxiliary for Fraiman method. Intends to speed calculations using R builtins.
+f.div <- function(x) { x[[1]]/x[[2]] }
+
+# Auxiliary for Fraiman method. Intends to speed calculations using R builtins.
+f.calcM <- function(x){ mapply(f.div, mapply(f.add, x, SIMPLIFY = F), SIMPLIFY = F)}
+
+## Auxiliary for Fraiman method. Padronize input.
+f.transform <- function(g)
+{
+  if(class(g)=='igraph'){ return(as.matrix(igraph::get.adjacency(g))) }
+  else if(class(g)=='list' && class(g[[1]])=='igraph'){
+    d <- lapply(g, f.transform)
+    return(d)
+  }
+  else if(class(g)=='list' && class(g[[1]])=='list'){
+    d <- lapply(g, f.transform)
+    return(d)
+  }
+}
+
+## Auxiliary for Fraiman method. Boostrap for the test.
+f.sampling.distribution <- function(g, maxPer = 300)
+{
+  
+  # creates a list with all the graphs
+  G <- list()
+  n <- length(g)
+  for(i in 1:n) G <- append(G, g[[i]])
+  m <- length(G)
+  
+  dist.boot = c()
+  # bootstrap
+  for (i_per in 1:maxPer){
+    G1 <- sample(G, m, replace=F)
+    #modification made on April 1s, 2019:
+    if(n==2){
+      l <- list(G1[1:floor(m/2)], G1[(floor(m/2)+1):m])
+    }
+    else{
+      l <- list(G1[1:floor(m/3)], G1[(floor(m/3)+1):(2*floor(m/3))], G1[((2*floor(m/3))+1):m])
+    }
+    ## original was:
+    # l <- list(G1[1:floor(m/2)], G1[(floor(m/2)+1):m])
+    
+    dist.boot[i_per] <- f.test(l)
+  }
+  return(dist.boot)
+}
+
 # Graph models  ----------------------------------------------------------------
 
 # Erdos-Renyi graph
